#include <config.h>
#include <getopt.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "params.h"

void print_usage()
{
    printf("Usage: nch <options> address port protocol_file\n"
            "Options are:\n"
            "--help\t\tprint this screen\n"
            "--listen (-l)\twork in server mode (not yet implemented)\n"
	    "-n\t\tnumber of connections to initiate - default 1\n"
            "-w\t\tDelay (in milliseconds) between connection initiations - default once\n"
            "--port beg:end\tSource port range to bind to\n"
            "--bind ip\tLocal IP address to bind to\n"
          );
}

struct params options={ 0 };

int parse_cmdline(int argc, char *argv[] )
{
    const static struct option long_options[]= {
        {"help", 0, 0, 1},
        {"listen", 0, 0, 'l'},
        {"port", 1, 0, 'p'},
        {"bind", 1, 0, 2},
        {0, 0, 0, 0}
    };
    const static char optionstr[]="ln:w:p:";
    int c;
    int option_index;

    options.num_connections=1;
    options.delay.tv_sec=0;
    options.delay.tv_usec=0;
    options.bind_addr.s_addr=INADDR_ANY;

    while( (c=getopt_long( argc, argv, optionstr, long_options, &option_index ))!=-1 ) {
        switch(c) {
        case 1:
            /* help */
            print_usage();
            exit(0);
            break;
        case 'l':
            /* Listen mode */
            options.server=1;
            break;
	case 'n':
	    /* Num connections */
	    options.num_connections=strtoul( optarg, NULL, 0);
	    break;
        case 'w':
            /* Delay */
            {
                unsigned int time=strtoul( optarg, NULL, 0);
                options.delay.tv_sec=time/1000;
                options.delay.tv_usec=(time%1000)*1000;
            }
            break;
        case 2:
            /* Source IP to bind to */
            if( inet_aton( optarg, &(options.bind_addr) )==0 ) {
                fprintf( stderr, "Error in parsing --bind parameter\n");
                exit(0);
            }
            break;
        case 'p':
            /* Source ports range */
            {
                char *seperator=strchr( optarg, ':' );

                if( seperator==NULL ) {
                    fprintf( stderr, "Error parsing arguments: --port does not have a range\n" );
                    exit(0);
                }

                *(seperator++)='\0';

                options.ports_start=strtoul( optarg, NULL, 10);
                options.ports_end=strtoul( seperator, NULL, 10);

                if( options.ports_start==0 || options.ports_end==0 ) {
                    fprintf( stderr, "Error parsing --port range\n" );
                    exit(0);
                }
            }
            break;
        }
    }

    if( !options.server ) {
        if( optind<=(argc-3) ) {
            struct addrinfo hints;
            struct addrinfo *res=NULL;
            int error;

            memset( &hints, 0, sizeof(hints) );
            hints.ai_family=PF_INET;
            hints.ai_socktype=SOCK_STREAM;
            if( (error=getaddrinfo( argv[optind], argv[optind+1], &hints, &res ))!=0 ) {
                fprintf(stderr, "Error in parsing host and port names: %s\n", gai_strerror(error) );

                exit(error);
            }

	    options.address=*(struct sockaddr_in *)res->ai_addr;
	    
	    options.protocol_file=argv[optind+2];
        } else { /* No IP or port */
            fprintf(stderr, "Must supply IP, port and protocol file\n");
            exit(1);
        }
    } else {
        /* Server mode not yet implemented */
        fprintf(stderr, "Server mode not yet implemented\n");
        exit(1);
    }
    return 0;
}

