//
// C++ Interface: eel_macro_helpers
//
// Description: 
//
//
// Author: Ryan Cunningham <ryan@cs.ucla.edu>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef EEL_MACRO_HELPERS_H
#define EEL_MACRO_HELPERS_H

// debugging and logging macros
//
//*** add better debugging/logging
#define EEL_SHOW_DEBUG_MESSAGES
//#define EEL_SHOW_DEBUG_API_CALLS

#ifdef EEL_SHOW_DEBUG_MESSAGES
#define EEL_DEBUG_MESSAGE(msg) printf( msg "\n" );
#define EEL_DEBUG_MESSAGE_IF(cond, msg) if (cond) printf( msg "\n" );
#define EEL_DEBUG_MESSAGE1(msg, a1) printf( msg "\n", a1 );
#define EEL_DEBUG_MESSAGE2(msg, a1, a2) printf( msg "\n", a1, a2 );
#define EEL_DEBUG_ERRORNO_MESSAGE(msg) printf( msg " errno = %d = %s\n", errno, strerror(errno));
#define EEL_CHECK_NULL(func, val) if ( val == NULL ) printf("Xalloc returned NULL at " #func "\n");
#else
#define EEL_DEBUG_MESSAGE(msg)
#define EEL_DEBUG_MESSAGE_IF(cond, msg)
#define EEL_DEBUG_MESSAGE1(msg, a1)
#define EEL_DEBUG_MESSAGE2(msg, a1, a2)
#define EEL_DEBUG_ERRORNO_MESSAGE(msg)
#define EEL_CHECK_NULL(func, val)
#endif // EEL_SHOW_DEBUG_MESSAGES
#ifdef EEL_SHOW_DEBUG_API_CALLS
#define EEL_DEBUG_API_CALL(func) printf( #func "\n");
#else
#define EEL_DEBUG_API_CALL(func)
#endif // EEL_SHOW_DEBUG_API_CALLS

//
// some timer helpers
//
#ifndef TIMEVAL_TO_MILLISECONDS
#define TIMEVAL_TO_MILLISECONDS(macro_arg_tv) \
  ( ((macro_arg_tv).tv_sec * 1000) + ((macro_arg_tv).tv_usec / 1000) )
#endif
#ifndef MILLISECONDS_TO_TIMEVAL
#define MILLISECONDS_TO_TIMEVAL(macro_arg_tv, macro_arg_milli) \
  do { (macro_arg_tv).tv_sec = ((macro_arg_milli)/1000); \
       (macro_arg_tv).tv_usec = (((macro_arg_milli)%1000)*1000); } while (0)
#endif
// these are near-duplicates from time.h -- not all systems have them
#define	TIMERCLEAR(tvp)		(tvp)->tv_sec = (tvp)->tv_usec = 0
#define	TIMERCMP(tvp, uvp, cmp)						\
	(((tvp)->tv_sec == (uvp)->tv_sec) ?				\
	    ((tvp)->tv_usec cmp (uvp)->tv_usec) :			\
	    ((tvp)->tv_sec cmp (uvp)->tv_sec))
#define	TIMERADD(tvp, uvp, vvp)						\
	do {								\
		(vvp)->tv_sec = (tvp)->tv_sec + (uvp)->tv_sec;		\
		(vvp)->tv_usec = (tvp)->tv_usec + (uvp)->tv_usec;	\
		if ((vvp)->tv_usec >= 1000000) {			\
			(vvp)->tv_sec++;				\
			(vvp)->tv_usec -= 1000000;			\
		}							\
	} while (0)
#define	TIMERSUB(tvp, uvp, vvp)						\
	do {								\
		(vvp)->tv_sec = (tvp)->tv_sec - (uvp)->tv_sec;		\
		(vvp)->tv_usec = (tvp)->tv_usec - (uvp)->tv_usec;	\
		if ((vvp)->tv_usec < 0) {				\
			(vvp)->tv_sec--;				\
			(vvp)->tv_usec += 1000000;			\
		}							\
	} while (0)
        
//
// some general helper macros
//
#define SAFE_FREE(pointer) \
  do { \
    if (pointer) { \
      free((void*)(pointer)); \
      (pointer) = NULL; \
    } } while (0)

#endif // EEL_MACRO_HELPERS_H
